#include "qimageutil.h"

void copyQImage(QImage &src, QImage &dest, int x, int y)
{
    int srcx, destx, srcy, desty;
    unsigned int *input, *output;
    for(srcy=0, desty = y; srcy < src.height(); ++srcy, ++desty){
            input = (unsigned int *)src.scanLine(srcy);
            output = (unsigned int *)dest.scanLine(desty);
            for(srcx=0, destx = x; srcx < src.width(); ++srcx, ++destx)
                output[destx] = input[srcx];
    }
}

void copyQImageWithAlpha(QImage &src, QImage &dest, int x, int y)
{
    int srcx, destx, srcy, desty;
    unsigned int *input, *output;
    int alpha, r, g, b;
    for(srcy=0, desty = y; srcy < src.height(); ++srcy, ++desty){
            input = (unsigned int *)src.scanLine(srcy);
            output = (unsigned int *)dest.scanLine(desty);
            for(srcx=0, destx = x; srcx < src.width(); ++srcx, ++destx){
                alpha = qAlpha(input[srcx]);
                if(alpha == 0){
                    ;
                }
                else if(alpha != 255){
                    float srcPercent = ((float)alpha)/255.0;
                    float destPercent = 1.0-srcPercent;
                    r = (int)((srcPercent*qRed(input[srcx])) + (destPercent*qRed(output[destx])));
                    g = (int)((srcPercent*qGreen(input[srcx])) + (destPercent*qGreen(output[destx])));
                    b = (int)((srcPercent*qBlue(input[srcx])) + (destPercent*qBlue(output[destx])));
                    output[destx] = qRgba(r, g, b, 255);
                }
                else
                    output[destx] = input[srcx];
            }
    }
}


void tileQImage(QImage &tile, QImage &dest)
{
    int sx, sy, dx, dy;
    unsigned int *srcData, *destData;

    for(sy=0, dy=0; dy < dest.height(); ++sy, ++dy){
        if(sy >= tile.height())
            sy = 0;
        srcData = (unsigned int *)tile.scanLine(sy);
        destData = (unsigned int *)dest.scanLine(dy);
        for(sx=0, dx=0; dx < dest.width(); ++sx, ++dx){
            if(sx >= tile.width())
                sx = 0;
            destData[dx] = srcData[sx];
        }
    }
}

void tileQImage(QImage &dest, int dx, int dy, int dw, int dh, QImage &src,
                int sx, int sy, int sw, int sh)
{
    unsigned int *srcData, *destData;
    int orig_sx = sx;
    int orig_sy = sy;
    int orig_dx = dx;
    //int orig_dy = dy;
    int sx2 = sx+sw-1;
    int sy2 = sy+sh-1;
    int dx2 = dx+dw-1;
    int dy2 = dy+dh-1;

    for(;dy < dy2; ++sy, ++dy){
        if(sy > sy2)
            sy = orig_sy;
        srcData = (unsigned int *)src.scanLine(sy);
        destData = (unsigned int *)dest.scanLine(dy);
        for(sx=orig_sx, dx=orig_dx; dx < dx2; ++sx, ++dx){
            if(sx > sx2)
                sx = orig_sx;
            destData[dx] = srcData[sx];
        }
    }
}

void copyQImageSecondaryAlpha(QImage &dest, int dx, int dy, int dw, int dh,
                              QImage &src, int sx, int sy, int sw, int sh)
{
    unsigned int *srcData, *destData;
    int orig_sx = sx;
    int orig_sy = sy;
    int orig_dx = dx;
    //int orig_dy = dy;
    int sx2 = sx+sw-1;
    int sy2 = sy+sh-1;
    int dx2 = dx+dw-1;
    int dy2 = dy+dh-1;

    int r, g, b, alpha;

    for(;dy < dy2; ++sy, ++dy){
        if(sy > sy2)
            sy = orig_sy;
        srcData = (unsigned int *)src.scanLine(sy);
        destData = (unsigned int *)dest.scanLine(dy);
        for(sx=orig_sx, dx=orig_dx; dx < dx2; ++sx, ++dx){
            if(sx > sx2)
                sx = orig_sx;
            r = qRed(destData[dx]);
            g = qGreen(destData[dx]);
            b = qBlue(destData[dx]);
            alpha = qAlpha(srcData[sx]);
            destData[dx] = qRgba(r, g, b, alpha);
        }
    }
}




